local is_win32 = SystemInfo:platform() == Idstring("WIN32")
local NOT_WIN_32 = not is_win32
local GRID_H_MUL = (NOT_WIN_32 and 6.9 or 6.95) / 8
local ITEMS_PER_COLUMN = 3

_G.RenameWithController = RenameWithController or class()
RenameWithController._path = ModPath

Hooks:Add('LocalizationManagerPostInit', 'LocalizationManagerPostInit_renameinventorywithacontroller', function(loc)
	local language_filename

	for _, filename in pairs(file.GetFiles(RenameWithController._path .. 'loc/')) do
		local str = filename:match('^(.*).json$')
		if str and Idstring(str) and Idstring(str):key() == SystemInfo:language():key() then
			language_filename = filename
			break
		end
	end

	if language_filename then
		loc:load_localization_file(RenameWithController._path .. 'loc/' .. language_filename)
	end
	loc:load_localization_file(RenameWithController._path .. 'loc/english.json', false)
end)

Hooks:PostHook(BlackMarketGui, "_setup", "BlackMarketGuiPostSetup_renameinventorywithacontroller", function(gui, is_start_page, component_data)
    local controller_rename_w = {
		prio = 4,
		btn = "BTN_START",
		pc_btn = "menu_respec_tree_all",
		name = "bm_menu_btn_wp_rename_item",
		callback = callback(gui, gui, "rename_item_with_gamepad_callback")
	}
    local btn_x = 10
	gui._btns["controller_rename_w"] = BlackMarketGuiButtonItem:new(gui._buttons, controller_rename_w, btn_x)

	local controller_rename_m = {
		prio = 4,
		btn = "BTN_START",
		pc_btn = "menu_respec_tree_all",
		name = "bm_menu_btn_m_rename_item",
		callback = callback(gui, gui, "rename_item_with_gamepad_callback")
	}
	gui._btns["controller_rename_m"] = BlackMarketGuiButtonItem:new(gui._buttons, controller_rename_m, btn_x)
end)

function BlackMarketGui:populate_weapon_category_new(data)
	managers.blackmarket:clear_temporary()
	managers.blackmarket:clear_preview_blueprint()

	local category = data.category
	local crafted_category = managers.blackmarket:get_crafted_category(category) or {}
	local last_weapon = table.size(crafted_category) == 1
	local last_unlocked_weapon = nil

	if not last_weapon then
		local category_size = table.size(crafted_category)

		for i, crafted in pairs(crafted_category) do
			if not managers.blackmarket:weapon_unlocked(crafted.weapon_id) then
				category_size = category_size - 1
			end
		end

		last_unlocked_weapon = category_size == 1
	end

	local hold_crafted_item = managers.blackmarket:get_hold_crafted_item()
	local currently_holding = hold_crafted_item and hold_crafted_item.category == category
	local max_items = data.override_slots and data.override_slots[1] * data.override_slots[2] or 9
	local max_rows = tweak_data.gui.WEAPON_ROWS_PER_PAGE or 3
	max_items = max_rows * (data.override_slots and data.override_slots[1] or 3)

	for i = 1, max_items do
		data[i] = nil
	end

	local guis_catalog = "guis/"
	local bundle_folder = nil
	local weapon_data = Global.blackmarket_manager.weapons
	local new_data = {}
	local start_i = data.start_i
	local crafted, unlocked, part_dlc_lock, color_tweak = nil

	for i, index in pairs(data.on_create_data) do
		crafted = crafted_category[index]

		if crafted then
			new_data = {
				ignore_slot = data.equip_weapon_cosmetics and data.equip_weapon_cosmetics.weapon_id ~= crafted.weapon_id,
				name = crafted.weapon_id,
				name_localized = managers.blackmarket:get_weapon_name_by_category_slot(category, index),
				raw_name_localized = managers.weapon_factory:get_weapon_name_by_factory_id(crafted.factory_id)
			}
			new_data.custom_name_text = not new_data.ignore_slot and managers.blackmarket:get_crafted_custom_name(category, index, true)
			new_data.category = category
			new_data.slot = index
			unlocked, part_dlc_lock = managers.blackmarket:weapon_unlocked_by_crafted(new_data.category, new_data.slot)
			new_data.unlocked = unlocked
			new_data.level = managers.blackmarket:weapon_level(crafted.weapon_id)
			new_data.can_afford = true
			new_data.equipped = crafted.equipped
			new_data.skill_based = weapon_data[crafted.weapon_id].skill_based
			new_data.skill_name = new_data.skill_based and "bm_menu_skill_locked_" .. new_data.name
			new_data.func_based = weapon_data[crafted.weapon_id].func_based
			new_data.price = managers.money:get_weapon_slot_sell_value(category, index)
			local bitmap_texture, bg_texture = managers.blackmarket:get_weapon_icon_path(crafted.weapon_id, crafted.cosmetics)
			new_data.bitmap_texture = bitmap_texture
			new_data.bg_texture = not new_data.ignore_slot and bg_texture
			new_data.customize_locked = crafted.customize_locked
			new_data.locked_name = crafted.locked_name
			new_data.name_color = new_data.locked_name and crafted.cosmetics and tweak_data.economy.rarities[tweak_data.blackmarket.weapon_skins[crafted.cosmetics.id].rarity or "common"].color
			new_data.stream = true
			new_data.akimbo_gui_data = tweak_data.weapon[crafted.weapon_id] and tweak_data.weapon[crafted.weapon_id].akimbo_gui_data
			new_data.comparision_data = not new_data.ignore_slot and managers.blackmarket:get_weapon_stats(category, index)
			new_data.global_value = part_dlc_lock or tweak_data.weapon[new_data.name] and tweak_data.weapon[new_data.name].global_value or "normal"

			if _G.IS_VR then
				new_data.vr_locked = tweak_data.vr:is_locked("weapons", crafted.weapon_id)
				new_data.unlocked = new_data.unlocked and not tweak_data.vr:is_locked("weapons", crafted.weapon_id)
			end

			new_data.dlc_locked = tweak_data.lootdrop.global_values[new_data.global_value].unlock_id or nil
			new_data.lock_texture = new_data.ignore_slot or self:get_lock_icon(new_data)
			new_data.holding = currently_holding and hold_crafted_item.slot == index
			new_data.part_dlc_lock = part_dlc_lock
			new_data.global_value = tweak_data.weapon[new_data.name] and tweak_data.weapon[new_data.name].global_value or "normal"

			if data.equip_weapon_cosmetics then
				new_data.equipped_slot = new_data.slot
				new_data.equipped_name = new_data.name
			end

			local icon_list = managers.menu_component:create_weapon_mod_icon_list(crafted.weapon_id, category, crafted.factory_id, index)
			local icon_index = 1
			local new_parts = {}

			for _, part in pairs(managers.blackmarket:get_weapon_new_part_drops(crafted.factory_id) or {}) do
				local type = tweak_data.weapon.factory.parts[part].type
				new_parts[type] = true
			end

			if table.size(new_parts) > 0 then
				new_data.new_drop_data = {}
			end

			new_data.mini_icons = {}

			if not new_data.ignore_slot then
				for _, icon in ipairs(icon_list) do
					table.insert(new_data.mini_icons, {
						layer = 1,
						h = 16,
						stream = false,
						w = 16,
						texture = icon.texture,
						right = (icon_index - 1) % 11 * 18,
						bottom = math.floor((icon_index - 1) / 11) * 25,
						alpha = icon.equipped and 1 or 0.25
					})

					if new_parts[icon.type] then
						table.insert(new_data.mini_icons, {
							texture = "guis/textures/pd2/blackmarket/inv_mod_new",
							layer = 1,
							h = 8,
							stream = false,
							w = 16,
							alpha = 1,
							right = (icon_index - 1) % 11 * 18,
							bottom = math.floor((icon_index - 1) / 11) * 25 + 16
						})
					end

					icon_index = icon_index + 1
				end
			end

			new_data.hide_unselected_mini_icons = true
			color_tweak = crafted.cosmetics and tweak_data.blackmarket.weapon_skins[crafted.cosmetics.id]

			if color_tweak and color_tweak.is_a_color_skin then
				guis_catalog = "guis/"
				bundle_folder = color_tweak.texture_bundle_folder

				if bundle_folder then
					guis_catalog = guis_catalog .. "dlcs/" .. tostring(bundle_folder) .. "/"
				end

				table.insert(new_data.mini_icons, {
					stream = true,
					h = 32,
					layer = 0,
					w = 64,
					right = -16,
					texture = guis_catalog .. "textures/pd2/blackmarket/icons/weapon_color/" .. crafted.cosmetics.id,
					bottom = math.floor((#icon_list - 1) / 11) * 25 + 24
				})
			end

			if not new_data.unlocked then
				new_data.last_weapon = last_weapon
			else
				new_data.last_weapon = last_weapon or last_unlocked_weapon
			end

			if new_data.equipped then
				self._equipped_comparision_data = self._equipped_comparision_data or {}
				self._equipped_comparision_data[category] = new_data.comparision_data
			end

			local active = true

			if not new_data.ignore_slot then
				if data.equip_weapon_cosmetics and data.equip_weapon_cosmetics.weapon_id == crafted.weapon_id then
					if active then
						table.insert(new_data, "wcc_equip")
					end

					new_data.equip_weapon_cosmetics = data.equip_weapon_cosmetics
				elseif currently_holding then
					new_data.selected_text = managers.localization:to_upper_text("bm_menu_btn_swap_weapon")

					if new_data.slot ~= hold_crafted_item.slot then
						table.insert(new_data, "w_swap")
					end

					table.insert(new_data, "i_stop_move")
				else
					local has_mods = managers.weapon_factory:has_weapon_more_than_default_parts(crafted.factory_id)
					local can_mod = true

					if can_mod and data.allow_modify ~= false and has_mods and active then
						table.insert(new_data, "w_mod")
					end

					if data.allow_skinning ~= false and has_mods and active and managers.workshop and managers.workshop:enabled() and not table.contains(managers.blackmarket:skin_editor():get_excluded_weapons(), new_data.name) then
						table.insert(new_data, "w_skin")
					end

					if data.allow_sell ~= false and not new_data.last_weapon then
						table.insert(new_data, "w_sell")
					end

					if active and not new_data.equipped and new_data.unlocked then
						table.insert(new_data, "w_equip")
					end

					if new_data.equipped and new_data.unlocked then
						table.insert(new_data, "w_move")
						table.insert(new_data, "controller_rename_w")
					end

					if data.allow_preview ~= false and active then
						table.insert(new_data, "w_preview")
					end
				end
			end

			data[i] = new_data
		end
	end

	for i = 1, max_items do
		if not data[i] then
			new_data = {}

			if data.allow_buy ~= false then
				local can_buy_weapon = managers.blackmarket:is_weapon_slot_unlocked(category, data.on_create_data[i])

				if can_buy_weapon then
					new_data.name = "bm_menu_btn_buy_new_weapon"
					new_data.name_localized = managers.localization:text("bm_menu_empty_weapon_slot")
					new_data.mid_text = {
						noselected_text = new_data.name_localized,
						noselected_color = tweak_data.screen_colors.button_stage_3
					}
					new_data.mid_text.selected_text = currently_holding and new_data.mid_text.noselected_text or managers.localization:text("bm_menu_btn_buy_new_weapon")
					new_data.mid_text.selected_color = currently_holding and new_data.mid_text.noselected_color or tweak_data.screen_colors.button_stage_2
					new_data.empty_slot = true
					new_data.category = category
					new_data.slot = data.on_create_data[i]
					new_data.unlocked = true
					new_data.equipped = false

					if data.equip_weapon_cosmetics then
						if managers.money:can_afford_weapon(data.equip_weapon_cosmetics.weapon_id) then
							table.insert(new_data, "wcc_buy_equip_weapon")

							new_data.equip_weapon_cosmetics = data.equip_weapon_cosmetics
						else
							new_data.mid_text.selected_text = managers.localization:text("bm_menu_cannot_buy_weapon_slot")
							new_data.mid_text.selected_color = tweak_data.screen_colors.important_1
							new_data.dlc_locked = managers.localization:to_upper_text("bm_menu_cannot_buy_weapon_slot")
							new_data.mid_text.lock_noselected_color = tweak_data.screen_colors.important_1
							new_data.cannot_buy = true
						end
					else
						if currently_holding then
							new_data.selected_text = managers.localization:to_upper_text("bm_menu_btn_place_weapon")

							table.insert(new_data, "w_place")
							table.insert(new_data, "i_stop_move")
						else
							table.insert(new_data, "ew_buy")
						end

						if managers.blackmarket:got_new_drop(new_data.category, "weapon_buy_empty", nil) then
							new_data.mini_icons = new_data.mini_icons or {}

							table.insert(new_data.mini_icons, {
								stream = false,
								name = "new_drop",
								h = 16,
								layer = 1,
								w = 16,
								texture = "guis/textures/pd2/blackmarket/inv_newdrop",
								visible = false,
								top = 0,
								right = 0
							})

							new_data.new_drop_data = {}
						end
					end
				else
					new_data.name = "bm_menu_btn_buy_weapon_slot"
					new_data.name_localized = managers.localization:text("bm_menu_locked_weapon_slot")
					new_data.empty_slot = true
					new_data.category = category
					new_data.slot = data.on_create_data[i]
					new_data.unlocked = true
					new_data.equipped = false
					new_data.lock_texture = "guis/textures/pd2/blackmarket/money_lock"
					new_data.lock_color = tweak_data.screen_colors.button_stage_3
					new_data.lock_shape = {
						w = 32,
						x = 0,
						h = 32,
						y = -32
					}
					new_data.locked_slot = true
					new_data.dlc_locked = managers.experience:cash_string(managers.money:get_buy_weapon_slot_price())
					new_data.mid_text = {
						noselected_text = new_data.name_localized,
						noselected_color = tweak_data.screen_colors.button_stage_3,
						is_lock_same_color = true
					}

					if currently_holding then
						new_data.mid_text.selected_text = new_data.mid_text.noselected_text
						new_data.mid_text.selected_color = new_data.mid_text.noselected_color

						table.insert(new_data, "i_stop_move")
					elseif managers.money:can_afford_buy_weapon_slot() then
						new_data.mid_text.selected_text = managers.localization:text("bm_menu_btn_buy_weapon_slot")
						new_data.mid_text.selected_color = tweak_data.screen_colors.button_stage_2

						table.insert(new_data, "ew_unlock")
					else
						new_data.mid_text.selected_text = managers.localization:text("bm_menu_cannot_buy_weapon_slot")
						new_data.mid_text.selected_color = tweak_data.screen_colors.important_1
						new_data.dlc_locked = new_data.dlc_locked .. "  " .. managers.localization:to_upper_text("bm_menu_cannot_buy_weapon_slot")
						new_data.mid_text.lock_noselected_color = tweak_data.screen_colors.important_1
						new_data.cannot_buy = true
					end
				end
			else
				new_data.name = "bm_menu_btn_unavailable"
				new_data.name_localized = managers.localization:text("bm_menu_btn_unavailable")
				new_data.mid_text = {
					noselected_text = new_data.name_localized,
					noselected_color = tweak_data.screen_colors.button_stage_3
				}
				new_data.mid_text.selected_text = currently_holding and new_data.mid_text.noselected_text or managers.localization:text("bm_menu_btn_unavailable")
				new_data.mid_text.selected_color = currently_holding and new_data.mid_text.noselected_color or tweak_data.screen_colors.button_stage_2
				new_data.empty_slot = true
				new_data.is_loadout = true
				new_data.category = category
				new_data.slot = data.on_create_data[i]
				new_data.unlocked = true
				new_data.equipped = false
			end

			data[i] = new_data
		end
	end
end

function BlackMarketGui:populate_masks_new(data)
	local new_data = {}
	local crafted_category = managers.blackmarket:get_crafted_category("masks") or {}
	local mini_icon_helper = math.round((self._panel:h() - (tweak_data.menu.pd2_small_font_size + 10) - 60) * GRID_H_MUL / ITEMS_PER_COLUMN) - 16
	local hold_crafted_item = managers.blackmarket:get_hold_crafted_item()
	local currently_holding = hold_crafted_item and hold_crafted_item.category == "masks"
	local max_items = data.override_slots and data.override_slots[1] * data.override_slots[2] or 9
	local max_rows = tweak_data.gui.MASK_ROWS_PER_PAGE or 3
	max_items = max_rows * (data.override_slots and data.override_slots[1] or 3)

	for i = 1, max_items do
		data[i] = nil
	end

	local guis_catalog = "guis/"
	local crafted = nil

	for i, index in pairs(data.on_create_data) do
		crafted = crafted_category[index]

		if crafted then
			local guis_mask_id = crafted.mask_id

			if tweak_data.blackmarket.masks[guis_mask_id].guis_id then
				guis_mask_id = tweak_data.blackmarket.masks[guis_mask_id].guis_id
			end

			new_data = {
				name = crafted.mask_id,
				name_localized = managers.blackmarket:get_mask_name_by_category_slot("masks", index)
			}
			new_data.raw_name_localized = managers.localization:text(tweak_data.blackmarket.masks[new_data.name].name_id)
			new_data.custom_name_text = managers.blackmarket:get_crafted_custom_name("masks", index, true)
			new_data.custom_name_text_right = crafted.modded and -55 or -20
			new_data.custom_name_text_width = crafted.modded and 0.6
			new_data.category = "masks"
			new_data.global_value = crafted.global_value
			new_data.slot = index
			new_data.unlocked = true
			new_data.equipped = crafted.equipped
			new_data.bitmap_texture = managers.blackmarket:get_mask_icon(guis_mask_id, self._data.character_id)
			new_data.stream = false
			new_data.holding = currently_holding and hold_crafted_item.slot == index
			new_data.item_id = crafted.item_id
			local dlc = tweak_data.blackmarket.masks[new_data.name].dlc or managers.dlc:global_value_to_dlc(new_data.global_value)

			if dlc and not managers.dlc:is_dlc_unlocked(dlc) then
				new_data.unlocked = false
				new_data.lock_texture = self:get_lock_icon(new_data, "guis/textures/pd2/lock_incompatible")
				local dlc_global_value = managers.dlc:dlc_to_global_value(dlc)
				new_data.dlc_locked = dlc_global_value and tweak_data.lootdrop.global_values[dlc_global_value] and tweak_data.lootdrop.global_values[dlc_global_value].unlock_id or "bm_menu_dlc_locked"
			elseif not managers.dlc:is_content_achievement_locked(data.category, new_data.name) then
				if managers.dlc:is_content_achievement_milestone_locked(data.category, new_data.name) then
					-- Nothing
				elseif managers.dlc:is_content_skirmish_locked(data.category, new_data.name) then
					-- Nothing
				elseif managers.dlc:is_content_crimespree_locked(data.category, new_data.name) then
					-- Nothing
				elseif managers.dlc:is_content_infamy_locked(data.category, new_data.name) then
					-- Nothing
				end
			end

			local locked_global_value = nil
			local locked_parts = {}

			if new_data.unlocked then
				local name_converter = {
					pattern = "textures",
					color = "colors",
					material = "materials"
				}
				local default_blueprint = tweak_data.blackmarket.masks[crafted.mask_id] and tweak_data.blackmarket.masks[crafted.mask_id].default_blueprint or {}

				for type, part in pairs(crafted.blueprint) do
					if default_blueprint[type] ~= part.id and default_blueprint[name_converter[type]] ~= part.id and tweak_data.lootdrop.global_values[part.global_value] and tweak_data.lootdrop.global_values[part.global_value].dlc and not managers.dlc:is_dlc_unlocked(part.global_value) then
						locked_parts[type] = part.global_value
						locked_global_value = part.global_value or locked_global_value
						new_data.unlocked = false
					end
				end
			else
				locked_global_value = new_data.global_value
			end

			if not new_data.unlocked then
				local t, gv = next(locked_parts)

				if gv then
					new_data.lock_texture = self:get_lock_icon(new_data, "guis/textures/pd2/lock_incompatible")
					new_data.dlc_locked = tweak_data.lootdrop.global_values[gv] and tweak_data.lootdrop.global_values[gv].unlock_id or "bm_menu_dlc_locked"
				end
			end

			new_data.active = true

			if currently_holding then
				if index ~= 1 then
					new_data.selected_text = managers.localization:to_upper_text("bm_menu_btn_swap_mask")
				end

				if index ~= 1 and new_data.slot ~= hold_crafted_item.slot then
					table.insert(new_data, "m_swap")
				end

				table.insert(new_data, "i_stop_move")
			else
				if new_data.unlocked and new_data.active then
					if not new_data.equipped then
						table.insert(new_data, "m_equip")
					end

					if index ~= 1 and new_data.equipped then
						table.insert(new_data, "m_move")
						table.insert(new_data, "controller_rename_m")
					end

					local can_mod_mask = true

					if can_mod_mask and not crafted.modded and managers.blackmarket:can_modify_mask(index) and index ~= 1 then
						table.insert(new_data, "m_mod")
					end

					table.insert(new_data, "m_preview")
				else
					if index ~= 1 and new_data.equipped then
						table.insert(new_data, "m_move")
					end

					if new_data.active then
						table.insert(new_data, "m_preview")
					end
				end

				if index ~= 1 then
					if managers.money:get_mask_sell_value(new_data.name, new_data.global_value) > 0 then
						table.insert(new_data, "m_sell")
					else
						table.insert(new_data, "m_remove")
					end

					if is_win32 and not managers.menu:is_pc_controller() then
						-- Nothing
					end
				end
			end

			if crafted.modded then
				new_data.mini_icons = {}
				local color_1 = tweak_data.blackmarket.mask_colors[crafted.blueprint.color_a.id].color
				local color_2 = tweak_data.blackmarket.mask_colors[crafted.blueprint.color_b.id].color

				table.insert(new_data.mini_icons, {
					texture = false,
					layer = 1,
					h = 16,
					w = 16,
					bottom = 0,
					right = 0,
					color = color_2
				})
				table.insert(new_data.mini_icons, {
					texture = false,
					layer = 1,
					h = 16,
					w = 16,
					bottom = 0,
					right = 18,
					color = color_1
				})

				if locked_parts.color then
					local texture = self:get_lock_icon({
						global_value = locked_parts.color
					})

					table.insert(new_data.mini_icons, {
						layer = 2,
						h = 24,
						w = 24,
						bottom = -4,
						right = 6,
						texture = texture,
						color = tweak_data.screen_colors.important_1
					})
				end

				local pattern = crafted.blueprint.pattern.id

				if pattern ~= "solidfirst" then
					if pattern ~= "solidsecond" then
						local material_id = crafted.blueprint.material.id
						guis_catalog = "guis/"
						local bundle_folder = tweak_data.blackmarket.materials[material_id] and tweak_data.blackmarket.materials[material_id].texture_bundle_folder

						if bundle_folder then
							guis_catalog = guis_catalog .. "dlcs/" .. tostring(bundle_folder) .. "/"
						end

						local right = 2
						local bottom = 38 - (NOT_WIN_32 and 10 or 10)
						local w = 32
						local h = 32

						table.insert(new_data.mini_icons, {
							layer = 1,
							stream = true,
							texture = guis_catalog .. "textures/pd2/blackmarket/icons/materials/" .. material_id,
							right = right,
							bottom = bottom,
							w = w,
							h = h
						})

						if locked_parts.material then
							local texture = self:get_lock_icon({
								global_value = locked_parts.material
							})

							table.insert(new_data.mini_icons, {
								layer = 2,
								texture = texture,
								w = w - 8,
								h = w - 8,
								right = right + 4,
								bottom = bottom + 4,
								color = tweak_data.screen_colors.important_1
							})
						end
					end
				end

				local right = 2
				local bottom = math.round(mini_icon_helper - 6 - 6 - 60 - 12)
				local w = 32
				local h = 32

				table.insert(new_data.mini_icons, {
					layer = 1,
					stream = true,
					texture = tweak_data.blackmarket.textures[pattern].texture,
					right = right,
					bottom = bottom,
					w = h,
					h = w,
					render_template = Idstring("VertexColorTexturedPatterns")
				})

				if locked_parts.pattern then
					local texture = self:get_lock_icon({
						global_value = locked_parts.pattern
					})

					table.insert(new_data.mini_icons, {
						layer = 2,
						texture = texture,
						w = w - 8,
						h = w - 8,
						right = right + 4,
						bottom = bottom + 4,
						color = tweak_data.screen_colors.important_1
					})
				end

				new_data.mini_icons.borders = true
			elseif index ~= 1 and managers.blackmarket:can_modify_mask(i) and managers.blackmarket:got_new_drop("normal", "mask_mods", crafted.mask_id) then
				new_data.mini_icons = new_data.mini_icons or {}

				table.insert(new_data.mini_icons, {
					stream = false,
					name = "new_drop",
					h = 16,
					layer = 1,
					w = 16,
					texture = "guis/textures/pd2/blackmarket/inv_newdrop",
					visible = true,
					top = 0,
					right = 0
				})

				new_data.new_drop_data = {}
			end

			if self._populate_masks_debug then
				self:_populate_masks_debug(crafted, new_data)
			end

			data[i] = new_data
		end
	end

	local can_buy_masks = true

	for i = 1, max_items do
		if not data[i] then
			can_buy_masks = managers.blackmarket:is_mask_slot_unlocked(data.on_create_data[i])
			new_data = {}

			if can_buy_masks then
				new_data.name = "bm_menu_btn_buy_new_mask"
				new_data.name_localized = managers.localization:text("bm_menu_empty_mask_slot")
				new_data.mid_text = {
					noselected_text = new_data.name_localized,
					noselected_color = tweak_data.screen_colors.button_stage_3
				}
				new_data.mid_text.selected_text = currently_holding and new_data.mid_text.noselected_text or managers.localization:text("bm_menu_btn_buy_new_mask")
				new_data.mid_text.selected_color = currently_holding and new_data.mid_text.noselected_color or tweak_data.screen_colors.button_stage_2
				new_data.empty_slot = true
				new_data.category = "masks"
				new_data.slot = data.on_create_data[i]
				new_data.unlocked = true
				new_data.equipped = false
				new_data.num_backs = 0
				new_data.cannot_buy = not can_buy_masks

				if currently_holding then
					if data.on_create_data[i] ~= 1 then
						new_data.selected_text = managers.localization:to_upper_text("bm_menu_btn_place_mask")
					end

					if data.on_create_data[i] ~= 1 then
						table.insert(new_data, "m_place")
					end

					table.insert(new_data, "i_stop_move")
				else
					table.insert(new_data, "em_buy")
				end

				if data.on_create_data[i] ~= 1 and managers.blackmarket:got_new_drop(nil, "mask_buy", nil) then
					new_data.mini_icons = new_data.mini_icons or {}

					table.insert(new_data.mini_icons, {
						stream = false,
						name = "new_drop",
						h = 16,
						layer = 1,
						w = 16,
						texture = "guis/textures/pd2/blackmarket/inv_newdrop",
						visible = false,
						top = 0,
						right = 0
					})

					new_data.new_drop_data = {}
				end
			else
				new_data.name = "bm_menu_btn_buy_mask_slot"
				new_data.name_localized = managers.localization:text("bm_menu_locked_mask_slot")
				new_data.empty_slot = true
				new_data.category = "masks"
				new_data.slot = data.on_create_data[i]
				new_data.unlocked = true
				new_data.equipped = false
				new_data.num_backs = 0
				new_data.lock_texture = "guis/textures/pd2/blackmarket/money_lock"
				new_data.lock_color = tweak_data.screen_colors.button_stage_3
				new_data.lock_shape = {
					w = 32,
					x = 0,
					h = 32,
					y = -32
				}
				new_data.locked_slot = true
				new_data.dlc_locked = managers.experience:cash_string(managers.money:get_buy_mask_slot_price())
				new_data.mid_text = {
					noselected_text = new_data.name_localized,
					noselected_color = tweak_data.screen_colors.button_stage_3,
					is_lock_same_color = true
				}

				if currently_holding then
					new_data.mid_text.selected_text = new_data.mid_text.noselected_text
					new_data.mid_text.selected_color = new_data.mid_text.noselected_color

					table.insert(new_data, "i_stop_move")
				elseif managers.money:can_afford_buy_mask_slot() then
					new_data.mid_text.selected_text = managers.localization:text("bm_menu_btn_buy_mask_slot")
					new_data.mid_text.selected_color = tweak_data.screen_colors.button_stage_2

					table.insert(new_data, "em_unlock")
				else
					new_data.mid_text.selected_text = managers.localization:text("bm_menu_cannot_buy_mask_slot")
					new_data.mid_text.selected_color = tweak_data.screen_colors.important_1
					new_data.dlc_locked = new_data.dlc_locked .. "  " .. managers.localization:to_upper_text("bm_menu_cannot_buy_mask_slot")
					new_data.mid_text.lock_noselected_color = tweak_data.screen_colors.important_1
					new_data.cannot_buy = true
				end
			end

			data[i] = new_data
		end
	end
end

function BlackMarketGui:rename_item_with_gamepad_callback(data)
	if not self._renaming_item then
		local custom_name = managers.blackmarket:get_crafted_custom_name(data.category, data.slot) or ""
		self._renaming_item = {
			category = data.category,
			slot = data.slot,
			custom_name = custom_name
		}
		self._ws:connect_keyboard(Input:keyboard())
		self._panel:enter_text(callback(self, self, "enter_text"))
		self._panel:key_press(callback(self, self, "key_press"))
		self._panel:key_release(callback(self, self, "key_release"))
		self._rename_caret = self._panel:rect({
			name = "caret",
			layer = 2,
			x = 0,
			y = 0,
			w = 0,
			h = 0,
			color = Color(0.05, 1, 1, 1)
		})
		self._rename_caret:animate(self.blink)
		self._caret_connected = true
		self:update_info_text()
	end
end